;**********************************************************************
;   This file is a basic code template for object module code         *
;   generation on the PIC16F628A. This file contains the              *
;   basic code building blocks to build upon.                         *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler and linker (Document DS33014).          *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:        xxx.asm                                         *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:                                                          *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: P16F628A.INC                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;**********************************************************************

    list      p=16F628A           ; list directive to define processor
    #include <p16F628A.inc>       ; processor specific variable definitions

    errorlevel  -302              ; suppress message 302 from list file

     __CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _RC_OSC_CLKOUT 

;portA
RA4_IN			EQU		0x04
RA5_IN			EQU		0x05

;portB
RB0_RX			EQU		0x00
RB1_MOT			EQU		0x01
RB2_MOT			EQU		0x02
RB3_MOT			EQU		0x03
RB4_LED			EQU		0x04
RB5_LED			EQU		0x05
RB7_CTS			EQU		0x07


;stav
TMR0_TICK		EQU		0x00
SPRACUJ			EQU		0x01
INIT			EQU		0x02
CAKAJ			EQU		0x03
CYKLU			EQU		0x04
VYB_CYKL		EQU		0x05
STOP_CYKL		EQU		0x06
PRESKOK			EQU		0x07

;stav2
ROT_LEFT		EQU		0x00
ROT_RIGHT		EQU		0x01
PSKOK			EQU		0x02
PER_KON			EQU		0x03
STRIEDAJ		EQU		0x04
DEC_POM			EQU		0x05
NEKON_L			EQU		0x06
NEKON_R			EQU		0x07

;rs232
KAL_OK			EQU		0x00
START_KAL		EQU		0x01
MERAM_KAL		EQU		0x02
DATA_GO			EQU		0x03
ZELENA			EQU		0x04
WRITE_BUF		EQU		0x05
NEVYKRESLUJ		EQU		0x06
RAZ				EQU		0x07


; example of using Shared Uninitialized Data Section
INT_VAR        UDATA_SHR     
w_temp         RES     1        ; variable used for context saving 
status_temp    RES     1        ; variable used for context saving

PREMENNE 		UDATA 0x20
stav			RES		1
stav2			RES		1
rs232		 	RES		1
cyklus_int		RES		1
maly_krok		RES		2
velky_krok		RES		2
prijaty_bajt	RES		1
rc_bajt			RES		1
rc_bajt2		RES		1
posun_bitov		RES		1
pocet_prij		RES		1
buf_adr			RES		1
buf_data		RES		1
eeadr_wr		RES		1
eedata_wr		RES		1
znak			RES		1
tmr0_val		RES		1
nastavenie		RES		1
cakatel			RES		2
cakatel_s		RES		2
poloha			RES		1
kombinacia		RES		1
trvanie			RES		1
cyklus			RES		16
c_polohy		RES		16		
p_zasobnik		RES		1
cyk_bits		RES		1
pomocna			RES		1
rotacia			RES		1
rot				RES		1
vstup1			RES		1
vstup2			RES		1
perioda			RES		1
maska			RES		1

;este je volnych 0 bajt pamate RAM
;test			RES	0


BUFF_BANK1		UDATA	0xA0
buffer1			RES		0x50

BUFF_BANK2		UDATA	0x120
buffer2			RES		0x30


;**********************************************************************
RESET_VECTOR    CODE   0x0000     ; processor reset vector
        GOTO    START             ; go to beginning of program
    
INT_VECTOR    CODE    0x0004      ; interrupt vector location

INTERRUPT

        movwf   w_temp            ; save off current W register contents
        movf    STATUS,w          ; move status register into W register
        movwf   status_temp       ; save off contents of STATUS register

		BTFSS	INTCON, T0IE
		GOTO	Preskoc_prerusenie_TMR0
		BTFSC	INTCON, T0IF
		CALL	Prerusenie_TMR0
Preskoc_prerusenie_TMR0

		BTFSC	PIR1, TMR1IF
		CALL	Prerusenie_TMR1

		BTFSS	INTCON, INTE
		GOTO	Preskoc_RB0_INT
		BTFSC	INTCON, INTF
		CALL	Prerusenie_RB0
Preskoc_RB0_INT

;		BTFSC	PIR1, EEIF
;		CALL	Prerusenie_EEPROM

        movf    status_temp,w     ; retrieve copy of STATUS register
        movwf   STATUS            ; restore pre-isr STATUS register contents
        swapf   w_temp,f
        swapf   w_temp,w          ; restore pre-isr W register contents
        retfie                    ; return from interrupt

MAIN_PROG     CODE

START
Resetni
		BANKSEL	PORTA
		CLRF	PORTA
		CLRF	PORTB
		BSF		PORTB, RB7_CTS
		
	;	MOVLW	0x07
	;	MOVWF	CMCON
		BCF		CMCON, C2INV
		BCF		CMCON, C1INV
		BSF		CMCON, CM2
		BCF		CMCON, CM1
		BCF		CMCON, CM0

		BANKSEL OPTION_REG
		MOVLW	0xC7
		MOVWF	OPTION_REG

		MOVLW	0xFF
		MOVWF	TRISA

		MOVLW	0x01
		MOVWF	TRISB
		
		MOVLW	0x01
		MOVWF	PIE1

		BANKSEL PIR1
		CLRF 	PIR1

		BCF		PORTB, RB1_MOT
		BCF		PORTB, RB2_MOT
		BCF		PORTB, RB3_MOT

		BCF		PORTB, RB4_LED
		BCF		PORTB, RB5_LED

		BTFSC	rs232, ZELENA
		BSF		PORTB, RB4_LED
		CLRF	rs232

		BSF		PORTB, RB7_CTS

		MOVLW	0x00
		MOVWF	T1CON
		CLRF	TMR1L
		CLRF	TMR1H

		CLRF	stav
		CLRF	stav2
		CLRF	pocet_prij
		CLRF	trvanie
		CLRF	kombinacia
		CLRF	vstup1
		CLRF	vstup2

		CLRF	p_zasobnik

		MOVLW	0x10
		MOVWF	pomocna
Vynuluj_zasobnik
		DECF	pomocna, W
		ADDLW	cyklus
		MOVWF	FSR
		CLRF	INDF
		DECF	pomocna, W
		ADDLW	c_polohy
		MOVWF	FSR
		CLRF	INDF
		DECFSZ	pomocna, F
		GOTO	Vynuluj_zasobnik

		MOVLW	0x04
		MOVWF	poloha

		CALL	Nastavenia_casovanie

		BCF		INTCON, T0IF
		BCF		INTCON, INTF
		BCF		PIR1, TMR1IF
		MOVLW	0xF0
		MOVWF	INTCON
		BCF		INTCON, T0IE		

Nekonecna_slucka
		;daj vediet ze sa moze zacat prenasat
		BCF		PORTB, RB7_CTS

		BANKSEL rs232
		BTFSS	rs232, WRITE_BUF
		GOTO	Preskoc_prekopirovanie_buffra_do_EEPROM
		BANKSEL	EECON1
		BTFSC	EECON1, WR
		GOTO	Preskoc_prekopirovanie_buffra_do_EEPROM
		;zakaz TMR1, TMR0 a ostatne
		BCF		INTCON, GIE
		BTFSC	INTCON, GIE
		GOTO	$-2
		BANKSEL PORTB
		BCF		rs232, WRITE_BUF
		BSF		PORTB, RB7_CTS
;zhasni LEDky
		BCF		PORTB, RB4_LED
		BCF		PORTB, RB5_LED
		CALL	Prekopiruj_z_buffra_do_EEPROM
;rozsviet na zeleno
		BSF		PORTB, RB4_LED
		BCF		PORTB, RB5_LED
		BCF		rs232, NEVYKRESLUJ
		CLRF	stav
		CLRF	rs232
		BSF		rs232, ZELENA
		CALL	Nastavenia_casovanie
		CLRF	trvanie
		CLRF	kombinacia
		MOVLW	0x04
		MOVWF	poloha
		;BSF		INTCON, GIE
		GOTO	Resetni
Preskoc_prekopirovanie_buffra_do_EEPROM

		BANKSEL stav
		BTFSS	stav, SPRACUJ
		GOTO	Preskoc_spracovanie_rx_bajtu
		MOVF	rc_bajt2, W
		CALL	Spracuj_prijaty_bajt
		BCF		stav, SPRACUJ
Preskoc_spracovanie_rx_bajtu

		BTFSS	rs232, RAZ
		GOTO	Preskoc_zhasnutie_LED
		BCF		rs232, RAZ
		BCF		PORTB, RB1_MOT
		BCF		PORTB, RB2_MOT
		BCF		PORTB, RB3_MOT
		CLRF	cakatel
		CLRF	cakatel+1
Preskoc_zhasnutie_LED

		BTFSS	rs232, NEVYKRESLUJ
		GOTO	Preskakuj_reset	
	;	BTFSC	PORTB, RB0_RX
	;	GOTO	Preskakuj_reset			
		BTFSS	PORTB, RB5_LED
		GOTO	Preskakuj_reset
		MOVLW	0x01
		ADDWF	cakatel, F
		BTFSS	STATUS, C
		GOTO	Preskakuj_reset		
		INCFSZ	cakatel+1, F
		GOTO	Preskakuj_reset
		GOTO	Resetni
Preskakuj_reset
;vykonavacia logika @@@@@@@@@@@@@@@@@@@@
		BTFSC	rs232, NEVYKRESLUJ
		GOTO 	Nekonecna_slucka
;daj vediet ze si zaneprazdneny
		BSF		PORTB, RB7_CTS
;spracuj casove udaje
		BTFSC	stav, VYB_CYKL
		GOTO	Motor_tocenie

		BTFSC	stav, PRESKOK
		GOTO	Motor_tocenie	

		BTFSC	INTCON, T0IE
		GOTO	Preskoc_nastavenie_TMR0
		BTFSC	stav, TMR0_TICK
		GOTO	Preskoc_nastavenie_TMR0
		BCF		INTCON, T0IF
		BSF		INTCON, T0IE
		BTFSC	stav2, ROT_LEFT
		GOTO	Nastav_pre_rotaciu
		BTFSC	stav2, ROT_RIGHT
		GOTO	Nastav_pre_rotaciu
		BTFSC	stav2, NEKON_L
		GOTO	Nastav_pre_rotaciu
		BTFSC	stav2, NEKON_R
		GOTO	Nastav_pre_rotaciu
		MOVF	tmr0_val, W
		MOVWF	TMR0
		GOTO	Preskoc_nastavenie_TMR0	
Nastav_pre_rotaciu
		MOVF	perioda, W
		MOVWF	TMR0
Preskoc_nastavenie_TMR0	

		BTFSS	stav, TMR0_TICK
		GOTO	Nekonecna_slucka
		BCF		stav, TMR0_TICK

		DECFSZ	cakatel, F
		GOTO	Preskoc_cakatel1
		MOVF	cakatel+1, F
		BTFSC	STATUS, Z
		GOTO	Odcitanie_trvanie
		DECFSZ	cakatel+1, F
		GOTO	Preskoc_cakatel1
		GOTO	Odcitanie_trvanie
Preskoc_cakatel1
		GOTO	Test_na_cakanie	

Odcitanie_trvanie
		MOVF	cakatel_s, W
		MOVWF	cakatel
		MOVF	cakatel_s+1, W
		MOVWF	cakatel+1
		MOVF	trvanie, F
		BTFSC	STATUS, Z
		GOTO	Preskoc_dalsie_ratanie
		DECFSZ	trvanie, F
		GOTO	Test_na_cakanie
Preskoc_dalsie_ratanie		
		BCF		stav, CAKAJ
		GOTO	Motor_tocenie
Test_na_cakanie
		BTFSC	stav, CAKAJ
		GOTO	Nekonecna_slucka

Motor_tocenie
		BCF		stav, VYB_CYKL
		BCF		stav, PRESKOK

		BTFSC	stav2, ROT_LEFT
		GOTO	Rotuj_dolava
		BTFSC	stav2, ROT_RIGHT
		GOTO	Rotuj_doprava

		MOVF	poloha, W
		CALL	Precitaj_znak_z_EEPROM
		MOVWF	kombinacia
		INCF	poloha, W
		CALL	Precitaj_znak_z_EEPROM	
		MOVWF	trvanie

		MOVF	kombinacia, W
		XORLW	0xFF
		BTFSS	STATUS, Z
		GOTO	Nie_je_koniec
		BSF		stav, PRESKOK
		GOTO	Nulovanie_polohy
Nie_je_koniec

		MOVF	kombinacia, W
		SUBLW	0x1F
		BTFSS	STATUS, C
		GOTO	Ina_hodnota
		MOVLW	0x10	
		SUBWF	kombinacia, W
		BTFSS	STATUS, C
		GOTO	Ina_hodnota	
		GOTO	Podmieneny_skok	
Ina_hodnota
		
		;MOVF	kombinacia, F
		;BTFSS	STATUS, Z
		;GOTO	Nezastav_rotaciu
		;BCF		stav2, NEKON_L
		;BCF		stav2, NEKON_R
		;GOTO	Preskoc_rotacie
;Nezastav_rotaciu
		BTFSC	stav2, NEKON_L
		GOTO	Rotuj_dolava
		BTFSC	stav2, NEKON_R
		GOTO	Rotuj_doprava
;Preskoc_rotacie


;prikaz na spracovanie cyklu
		BTFSS	kombinacia, 0x07
		GOTO	Preskoc_cyklenie
		BTFSS	stav, CYKLU
		GOTO	Nastav_cyklus
;skontroluj ci uz nebezi nejaky cyklus
		MOVLW	c_polohy
		ADDWF	p_zasobnik, W
		MOVWF	FSR
		MOVF	INDF, W
		XORWF	poloha, W
		BTFSC	STATUS, Z
		GOTO	Rovnaky_cyklus
;ak je to novy cyklus
		MOVF	p_zasobnik, W
		XORLW	0x0F
		BTFSS	STATUS, Z
		INCF	p_zasobnik, F
		GOTO	Nastav_cyklus
Rovnaky_cyklus
		MOVLW	cyklus
		ADDWF	p_zasobnik, W
		MOVWF	FSR
		MOVF	INDF, W
		MOVWF	pomocna
		DECFSZ	pomocna, F
		GOTO	Preskoc_cyklus
		MOVLW	cyklus
		ADDWF	p_zasobnik, W
		MOVWF	FSR
		MOVF	pomocna, W
		MOVWF	INDF
		MOVF	p_zasobnik, F
		BTFSC	STATUS, Z
		BCF		stav, CYKLU		
		BTFSS	STATUS, Z
		DECF	p_zasobnik, F
		BSF		stav, STOP_CYKL
		BSF		stav, VYB_CYKL
		GOTO	Preskoc_cyklenie
Nastav_cyklus
;zapis pocet cyklov do zasobnika
		MOVLW	cyklus
		ADDWF	p_zasobnik, W
		MOVWF	FSR
		MOVF	trvanie, W
		MOVWF	INDF ;cyklus
		CLRF	trvanie
;zapis polohu zaciatku cyklu do zasobnika
		MOVLW	c_polohy
		ADDWF	p_zasobnik, W
		MOVWF	FSR
		MOVF	poloha, W
		MOVWF	INDF
		BSF		stav, CYKLU
		GOTO	Preskoc_zapis_do_zasobnika
Preskoc_cyklus
		MOVLW	cyklus
		ADDWF	p_zasobnik, W
		MOVWF	FSR
		MOVF	pomocna, W
		MOVWF	INDF
Preskoc_zapis_do_zasobnika
		MOVF	kombinacia, W
		MOVWF	poloha
		BCF		poloha, 0x07
		BSF		stav, VYB_CYKL
Preskoc_cyklenie

		BTFSC	stav, STOP_CYKL
		GOTO	Posun_citania
		BTFSC	stav, VYB_CYKL
		GOTO	Nekonecna_slucka 

;test na prikaz posuvu
;posuv dolava
		BTFSS	kombinacia, 0x06
		GOTO	Preskoc_rotaciu_dolava	
		BSF		stav2, ROT_LEFT
		BCF		stav2, PER_KON
		BCF		stav2, STRIEDAJ
		BCF		stav2, DEC_POM
		BCF		stav2, NEKON_L
		CLRF	perioda
		MOVLW	0x02
		MOVWF	PORTB
		MOVWF	rotacia
		BCF		kombinacia, 0x06
		BCF		kombinacia, 0x05
		BTFSC	kombinacia, 0x04
		BSF		stav2, NEKON_L
		BCF		kombinacia, 0x04 
		MOVF	kombinacia, W
		MOVWF	pomocna
		MOVF	trvanie, w
		MOVWF	kombinacia
		MOVF	pomocna, F
		BTFSC	STATUS, Z
		GOTO	Preskoc_delenie_left
Delenie_left
		BCF		STATUS, C
		RRF		kombinacia, F
		DECFSZ	pomocna, F
		GOTO	Delenie_left
Preskoc_delenie_left
		MOVLW	0xFF
		MOVWF	rot
		MOVF	trvanie, W
		MOVWF	pomocna
Rotuj_dolava
		BTFSC	stav2, STRIEDAJ
		GOTO	Vymaz_chvost_left
		BCF		STATUS, C		
		RLF		rotacia, F
		GOTO	Nemaz_chvost_left
Vymaz_chvost_left
		COMF	maska, W
		ANDWF	rotacia, F
		BCF		stav2, STRIEDAJ
		GOTO	Preskoc_prekrytie_left			
Nemaz_chvost_left
		MOVF	PORTB, W
		MOVWF	maska
		IORWF	rotacia, F
		BSF		stav2, STRIEDAJ
Preskoc_prekrytie_left
		BTFSS	rotacia, 0x04
		GOTO	Preskoc_prepis_pozicie1
		BCF		rotacia, 0x04
		BSF		rotacia, 0x01
Preskoc_prepis_pozicie1
		BTFSS	rotacia, 0x01
		BCF		PORTB, 0x01
		BTFSS	rotacia, 0x02
		BCF		PORTB, 0x02
		BTFSS	rotacia, 0x03
		BCF		PORTB, 0x03
		BTFSC	rotacia, 0x01
		BSF		PORTB, 0x01
		BTFSC	rotacia, 0x02
		BSF		PORTB, 0x02
		BTFSC	rotacia, 0x03
		BSF		PORTB, 0x03
		BSF		stav, CAKAJ
		BTFSC	stav2, DEC_POM
		GOTO	Dec_pom_left
		INCF	perioda, F
		MOVF	perioda, F
		BTFSS	STATUS, Z
		GOTO	Preskoc_vynuluj_periodu_left
		MOVLW	0xFF
		MOVWF	perioda
		BSF		stav2, DEC_POM	
Dec_pom_left
		MOVF	pomocna, F
		BTFSC	STATUS, Z
		GOTO	Pom_nul_left
		BTFSC	stav2, PER_KON
		GOTO	Preskoc_vynuluj_periodu_left
		MOVF	pomocna, W
		SUBWF	kombinacia, W
		BTFSC	STATUS, C
		GOTO	Pom_nul_left		
		DECFSZ	pomocna, F
		GOTO	Preskoc_vynuluj_periodu_left
Pom_nul_left
		BSF		stav2, PER_KON
Preskoc_vynuluj_periodu_left
		MOVF	pomocna, W
		MOVWF	trvanie
		BTFSS	stav2, PER_KON
		GOTO	Nekonecna_slucka
		MOVF	rot, F
		BTFSC	STATUS, Z
		GOTO	Koniec_rot_left
		DECFSZ	rot, F
		GOTO	Nekonecna_slucka
Koniec_rot_left
		MOVLW	0x0E
		ANDWF	PORTB, W
		XORLW	0x02
		BTFSS	STATUS, Z
		GOTO	Nekonecna_slucka
		BTFSS	stav2, ROT_LEFT	
		GOTO	Nekonecna_slucka
		BCF		stav2, ROT_LEFT
		GOTO	Posun_citania
Preskoc_rotaciu_dolava

;posuv doprava
		BTFSS	kombinacia, 0x05
		GOTO	Preskoc_rotaciu_doprava
		BSF		stav2, ROT_RIGHT
		BCF		stav2, PER_KON	
		BCF		stav2, STRIEDAJ
		BCF		stav2, DEC_POM
		BCF		stav2, NEKON_R
		CLRF	perioda
		MOVLW	0x02
		MOVWF	PORTB
		MOVWF	rotacia
		BCF		kombinacia, 0x06
		BCF		kombinacia, 0x05
		BTFSC	kombinacia, 0x04
		BSF		stav2, NEKON_R
		BCF		kombinacia, 0x04
		MOVF	kombinacia, W
		MOVWF	pomocna
		MOVF	trvanie, w
		MOVWF	kombinacia
		MOVF	pomocna, F
		BTFSC	STATUS, Z
		GOTO	Preskoc_delenie_right
Delenie_right
		BCF		STATUS, C
		RRF		kombinacia, F
		DECFSZ	pomocna, F
		GOTO	Delenie_right
Preskoc_delenie_right
		MOVLW	0xFF
		MOVWF	rot
		MOVF	trvanie, W
		MOVWF	pomocna
Rotuj_doprava
		BTFSC	stav2, STRIEDAJ
		GOTO	Vymaz_chvost_right
		BCF		STATUS, C		
		RRF		rotacia, F
		GOTO	Nemaz_chvost_right
Vymaz_chvost_right
		COMF	maska, W
		ANDWF	rotacia, F
		BCF		stav2, STRIEDAJ
		GOTO	Preskoc_prekrytie_right			
Nemaz_chvost_right
		MOVF	PORTB, W
		MOVWF	maska
		IORWF	rotacia, F
		BSF		stav2, STRIEDAJ
Preskoc_prekrytie_right
		BTFSS	rotacia, 0x00
		GOTO	Preskoc_prepis_pozicie2
		BCF		rotacia, 0x00
		BSF		rotacia, 0x03
Preskoc_prepis_pozicie2
		BTFSS	rotacia, 0x01
		BCF		PORTB, 0x01
		BTFSS	rotacia, 0x02
		BCF		PORTB, 0x02
		BTFSS	rotacia, 0x03
		BCF		PORTB, 0x03
		BTFSC	rotacia, 0x01
		BSF		PORTB, 0x01
		BTFSC	rotacia, 0x02
		BSF		PORTB, 0x02
		BTFSC	rotacia, 0x03
		BSF		PORTB, 0x03
		BSF		stav, CAKAJ
		BTFSC	stav2, DEC_POM
		GOTO	Dec_pom_right
		INCF	perioda, F
		MOVF	perioda, F
		BTFSS	STATUS, Z
		GOTO	Preskoc_vynuluj_periodu_right
		MOVLW	0xFF
		MOVWF	perioda
		BSF		stav2, DEC_POM	
Dec_pom_right
		MOVF	pomocna, F
		BTFSC	STATUS, Z
		GOTO	Pom_nul_right
		BTFSC	stav2, PER_KON
		GOTO	Preskoc_vynuluj_periodu_right
		MOVF	pomocna, W
		SUBWF	kombinacia, W
		BTFSC	STATUS, C
		GOTO	Pom_nul_right		
		DECFSZ	pomocna, F
		GOTO	Preskoc_vynuluj_periodu_right
Pom_nul_right
		BSF		stav2, PER_KON	
Preskoc_vynuluj_periodu_right
		MOVF	pomocna, W
		MOVWF	trvanie
		BTFSS	stav2, PER_KON
		GOTO	Nekonecna_slucka
		MOVF	rot, F
		BTFSC	STATUS, Z
		GOTO	Koniec_rot_right
		DECFSZ	rot, F
		GOTO	Nekonecna_slucka
Koniec_rot_right
		MOVLW	0x0E
		ANDWF	PORTB, W
		XORLW	0x02
		BTFSS	STATUS, Z
		GOTO	Nekonecna_slucka
		BTFSS	stav2, ROT_RIGHT
		GOTO	Nekonecna_slucka
		BCF		stav2, ROT_RIGHT
		GOTO	Posun_citania
Preskoc_rotaciu_doprava	

;podmieneny skok - vetvenie
Podmieneny_skok
		BTFSS	kombinacia, 0x04
		GOTO	Preskoc_vetvu
		CLRF	vstup1
		CLRF	vstup2
		BCF		stav2, PSKOK
		BTFSC	kombinacia, 0x03
		GOTO	Rozhoduj_podla_komparatorov
;digitalne vstupy
		BTFSC	kombinacia, 0x02
		GOTO	Negovane_digitalne_vstupy
;norm
		BTFSC	PORTA, RA4_IN
		BSF		vstup1, 0x00
		BTFSC	PORTA, RA5_IN
		BSF		vstup2, 0x00	
		GOTO	Preskoc_vetvenie
Negovane_digitalne_vstupy
;neg
		BTFSS	PORTA, RA4_IN
		BSF		vstup1, 0x00
		BTFSS	PORTA, RA5_IN
		BSF		vstup2, 0x00
		GOTO	Preskoc_vetvenie
Rozhoduj_podla_komparatorov
;komparatory
		BTFSC	kombinacia, 0x02
		GOTO	Negovane_komparatory
;norm
		MOVF	CMCON, F
		BTFSC	CMCON, C1OUT
		BSF		vstup1, 0x00
		BTFSC	CMCON, C2OUT
		BSF		vstup2, 0x00
		GOTO	Preskoc_vetvenie
Negovane_komparatory
;neg
		MOVF	CMCON, F
		BTFSS	CMCON, C1OUT
		BSF		vstup1, 0x00
		BTFSS	CMCON, C2OUT
		BSF		vstup2, 0x00
Preskoc_vetvenie

		BTFSC	kombinacia, 0x01
		GOTO	Urob_OR_NOR
;AND_NAND
		BTFSC	kombinacia, 0x00
		GOTO	Urob_NAND
;AND
		MOVF	vstup1, W
		ANDWF	vstup2, F
		BTFSC	vstup2, 0x00
		BSF		stav2, PSKOK
		GOTO	Preskoc_prikaz_vetvenie
Urob_NAND
;NAND
		MOVF	vstup1, W
		ANDWF	vstup2, F
		BTFSS	vstup2, 0x00
		BSF		stav2, PSKOK
		GOTO	Preskoc_prikaz_vetvenie
Urob_OR_NOR
		BTFSC	kombinacia, 0x00	
		GOTO	Urob_NOR
;OR
		MOVF	vstup1, W
		IORWF	vstup2, F
		BTFSC	vstup2, 0x00
		BSF		stav2, PSKOK
		GOTO	Preskoc_prikaz_vetvenie
Urob_NOR
;NOR
		MOVF	vstup1, W
		IORWF	vstup2, F
		BTFSS	vstup2, 0x00
		BSF		stav2, PSKOK
Preskoc_prikaz_vetvenie
		BCF		stav, CAKAJ
		BSF		stav, PRESKOK
		BTFSS	stav2, PSKOK
		GOTO	Nerob_podmieneny_skok
;podmieneny skok
		BCF		stav2, NEKON_L
		BCF		stav2, NEKON_R
		MOVF	trvanie, W
		MOVWF	poloha
		GOTO	Nekonecna_slucka
Nerob_podmieneny_skok
;bez skoku
		BTFSC	stav2, NEKON_L
		GOTO	Neposuvaj_citanie
		BTFSC	stav2, NEKON_R
		GOTO	Neposuvaj_citanie
		GOTO	Posun_citania
Neposuvaj_citanie
		BSF		stav, CAKAJ
		BCF		stav, PRESKOK
		GOTO	Ina_hodnota
Preskoc_vetvu
		


;zmena konfiguracie
		BTFSS	kombinacia, 0x03
		GOTO	Preskoc_konfiguraciu
		MOVLW	0x07
		IORWF	nastavenie, F
		BANKSEL OPTION_REG
		IORWF	OPTION_REG
		BANKSEL kombinacia
		MOVLW	0xF8
		IORWF	kombinacia, W
		ANDWF	nastavenie, F
		BANKSEL OPTION_REG
		ANDWF	OPTION_REG, F
		BANKSEL trvanie
		MOVF	trvanie, W
		MOVWF	tmr0_val
		CLRF	trvanie
		BCF		stav, CAKAJ
		BSF		stav, PRESKOK
		GOTO	Posun_citania
Preskoc_konfiguraciu


;spracovanie udajov z EEPROM pre motor
		BTFSS	kombinacia, 0x00
		BCF		PORTB, RB1_MOT
		BTFSS	kombinacia, 0x01
		BCF		PORTB, RB2_MOT
		BTFSS	kombinacia, 0x02
		BCF		PORTB, RB3_MOT

		BTFSC	kombinacia, 0x00
		BSF		PORTB, RB1_MOT
		BTFSC	kombinacia, 0x01
		BSF		PORTB, RB2_MOT
		BTFSC	kombinacia, 0x02
		BSF		PORTB, RB3_MOT

		BSF		stav, CAKAJ
Posun_citania
		BCF		stav, STOP_CYKL

		MOVLW	0x02
		ADDWF	poloha, F

		MOVF	poloha, W
		SUBLW	0x7F ;127
		BTFSC	STATUS, C
		GOTO	Preskoc_nulovanie_polohy
Nulovanie_polohy
		MOVLW	0x04
		MOVWF	poloha
Preskoc_nulovanie_polohy

GOTO Nekonecna_slucka                


Spracuj_prijaty_bajt
		MOVWF	prijaty_bajt
		XORLW	0xFE
		BTFSS	STATUS, Z
		GOTO	Preskoc_inicializaciu_bajtu
		BSF		stav, INIT
		BCF		rs232, WRITE_BUF
		CLRF	pocet_prij
		GOTO	Inicializacia_ok
Preskoc_inicializaciu_bajtu
		BTFSS	stav, INIT
		GOTO	Zahod_bajt
		MOVF	prijaty_bajt, W
		XORLW	0xFF
		BTFSS	STATUS, Z
		GOTO	Preskoc_ukoncenie_prijmu
		BCF		stav, INIT
		BSF		rs232, WRITE_BUF
		CALL	Zapis_do_buffra
		GOTO	Koniec_obsluhy_prijmu
Preskoc_ukoncenie_prijmu
		CALL	Zapis_do_buffra
		INCF	pocet_prij, F
		MOVF	pocet_prij, W
		XORLW	0x80 ;128
		BTFSS	STATUS, Z
		GOTO	Koniec_obsluhy_prijmu
		BCF		stav, INIT
		BSF		rs232, WRITE_BUF
Inicializacia_ok
Zahod_bajt	
Koniec_obsluhy_prijmu	
RETURN


Zapis_do_buffra
		BANKSEL	pocet_prij
		MOVF	pocet_prij, W
		SUBLW	0x4F ;79
		BTFSS	STATUS, C
		GOTO	Zapis_do_buffera2
		BANKSEL buffer1
		MOVLW	buffer1
		BANKSEL pocet_prij
		ADDWF	pocet_prij, W
		MOVWF	FSR
		MOVF	prijaty_bajt, W
		BANKSEL buffer1
		MOVWF	INDF
		GOTO	Koniec_zapisu_do_buffera
Zapis_do_buffera2
		BANKSEL pocet_prij
		MOVLW	0x50
		SUBWF	pocet_prij, W
		BSF		STATUS, IRP ;banky 2 - 3
		BANKSEL buffer2
		ADDLW	buffer2
		MOVWF	FSR
		BCF		STATUS, IRP ;banky 0 - 1
		BANKSEL prijaty_bajt
		MOVF	prijaty_bajt, W
		BSF		STATUS, IRP ;banky 2 - 3
		BANKSEL buffer2
		MOVWF	INDF
Koniec_zapisu_do_buffera
		BCF		STATUS, IRP ;banky 0 - 1	
		BANKSEL prijaty_bajt
RETURN


Prekopiruj_z_buffra_do_EEPROM
;		BANKSEL	rs232
;		BSF		rs232, EE_WR_OK
		CLRF	buf_adr
Kopirujem_data
		MOVF	buf_adr, W
		SUBLW	0x4F ;79
		BTFSS	STATUS, C
		GOTO	Citaj_z_buffera2
		BANKSEL buffer1
		MOVLW	buffer1
		BANKSEL buf_adr
		ADDWF	buf_adr, W
		MOVWF	FSR
		BANKSEL buffer1
		MOVF	INDF, W
		BANKSEL buf_data
		MOVWF	buf_data
		GOTO	Koniec_citania_z_buffera
Citaj_z_buffera2
		BANKSEL buf_adr
		MOVLW	0x50
		SUBWF	buf_adr, W
		BSF		STATUS, IRP ;banky 2 - 3
		BANKSEL buffer2
		ADDLW	buffer2
		MOVWF	FSR
		MOVF	INDF, W
		BCF		STATUS, IRP ;banky 0 - 1
		BANKSEL buf_data
		MOVWF	buf_data
Koniec_citania_z_buffera

		MOVF	buf_adr, W
		MOVWF	eeadr_wr
		MOVF	buf_data, W
		MOVWF	eedata_wr
Cakam_na_povolenie
	;	CLRWDT
		BANKSEL EECON1
		BTFSC	EECON1, WR
		GOTO	Cakam_na_povolenie
		BANKSEL stav
		CALL	Zapis_znak_do_EEPROM

;		BANKSEL	rs232
;		BCF		rs232, EE_WR_OK
		BANKSEL buf_adr
		INCF	buf_adr, F

		MOVF	buf_adr, W
		XORLW	0x80 ;128
		BTFSC	STATUS, Z
		GOTO	Ukonci_kopirovanie

		MOVF	buf_data, W
		XORLW	0xFF
		BTFSS	STATUS, Z
		GOTO	Kopirujem_data

		MOVF	buf_adr, W
		MOVWF	eeadr_wr
		MOVF	buf_data, W
		MOVWF	eedata_wr

Cakam_na_povolenie3
	;	CLRWDT
		BANKSEL EECON1
		BTFSC	EECON1, WR
		GOTO	Cakam_na_povolenie3
		BANKSEL stav
		CALL	Zapis_znak_do_EEPROM		

Ukonci_kopirovanie

Cakam_na_povolenie2
	;	CLRWDT
		BANKSEL EECON1
		BTFSC	EECON1, WR
		GOTO	Cakam_na_povolenie2
		BANKSEL stav
		;CALL	Najdi_ukoncenie	
RETURN


Zapis_znak_do_EEPROM
		BANKSEL eeadr_wr
		MOVF	eeadr_wr, W
		BANKSEL	EEADR
		MOVWF	EEADR
		BANKSEL eedata_wr
		MOVF	eedata_wr, W
		BANKSEL EEDATA
		MOVWF	EEDATA
		BANKSEL EECON1
		BSF		EECON1, WREN
		BCF		INTCON, GIE
		BTFSC	INTCON, GIE
		GOTO	$-2
		MOVLW	0x55
		MOVWF	EECON2
		MOVLW	0xAA
		MOVWF	EECON2
		BSF		EECON1, WR
		;BSF		INTCON, GIE
		BCF		EECON1, WREN
		BANKSEL	eeadr_wr
RETURN


Precitaj_znak_z_EEPROM
		BANKSEL EEADR
		MOVWF	EEADR
		BSF		EECON1, RD
		MOVF	EEDATA, W
		BANKSEL	znak
		MOVWF	znak
RETURN


Nastavenia_casovanie
		MOVLW	0x00
		CALL	Precitaj_znak_z_EEPROM
		MOVWF	tmr0_val
		MOVWF	TMR0

		MOVLW	0x01
		CALL	Precitaj_znak_z_EEPROM
		MOVWF	cakatel_s
		MOVWF	cakatel

		MOVLW	0x02
		CALL	Precitaj_znak_z_EEPROM
		MOVWF	cakatel_s+1
		MOVWF	cakatel+1

		MOVLW	0x03
		CALL	Precitaj_znak_z_EEPROM
		MOVWF	nastavenie
		BANKSEL OPTION_REG
		BCF		OPTION_REG, 0x00
		BCF		OPTION_REG, 0x01
		BCF		OPTION_REG, 0x02
		BANKSEL nastavenie
		BTFSS	nastavenie, 0x00
		GOTO	Preskoc_bit0
		BANKSEL OPTION_REG
		BSF		OPTION_REG, 0x00
Preskoc_bit0
		BANKSEL nastavenie
		BTFSS	nastavenie, 0x01
		GOTO	Preskoc_bit1
		BANKSEL OPTION_REG
		BSF		OPTION_REG, 0x01
Preskoc_bit1	
		BANKSEL nastavenie
		BTFSS	nastavenie, 0x02
		GOTO	Preskoc_bit2
		BANKSEL OPTION_REG
		BSF		OPTION_REG, 0x02
Preskoc_bit2
		BANKSEL nastavenie
RETURN

;##############################################################
;####################### PRERUSENIA ###########################
;##############################################################

Prerusenie_RB0
		BCF		INTCON, INTF

		BANKSEL rs232
		BTFSC	rs232, DATA_GO
		GOTO	Koniec_rc

		BTFSS	rs232, KAL_OK
		GOTO	Preskoc_kal_ok
		MOVF	maly_krok, W
		MOVWF	TMR1L
		MOVF	maly_krok+1, W
		MOVWF	TMR1H
		MOVLW	0x08
		MOVWF	posun_bitov
		BSF		rs232, DATA_GO
		BCF		INTCON, INTE
		BCF		PIR1, TMR1IF
		BSF		T1CON, TMR1ON
;rozsviet na cerveno
		BCF		PORTB, RB4_LED
		BSF		PORTB, RB5_LED
		CLRF	cakatel+1
		GOTO	Koniec_rc
Preskoc_kal_ok

		BTFSC	rs232, START_KAL
		GOTO	Preskoc_kalibraciu
		BTFSS	PORTB, RB0_RX
		GOTO	Koniec_rc
		CLRF	TMR1L
		CLRF	TMR1H
		BSF		T1CON, TMR1ON
;zhasni LEDky
		BCF		PORTB, RB4_LED
		BCF		PORTB, RB5_LED
		;CLRF	pocet_prij
		BSF		rs232, NEVYKRESLUJ
		BSF		rs232, RAZ
		BANKSEL OPTION_REG
		BCF		OPTION_REG, INTEDG
		BANKSEL	rs232
		BSF		rs232, START_KAL
		GOTO	Koniec_rc
Preskoc_kalibraciu	

		BTFSC	rs232, MERAM_KAL
		GOTO	Preskoc_vypocet_kroku
		BTFSS	PORTB, RB0_RX
		GOTO	Preskoc_chybu
;rozsviet na cerveno
		BCF		PORTB, RB4_LED
		BSF		PORTB, RB5_LED
Koniec_chyba_0x01	
		GOTO	Koniec_chyba_0x01
Preskoc_chybu
		BANKSEL OPTION_REG
		BSF		OPTION_REG, INTEDG
		BANKSEL	T1CON
		BCF		T1CON, TMR1ON
		MOVF	TMR1L, W
		MOVWF	velky_krok
		MOVF	TMR1H, W
		MOVWF	velky_krok+1
		MOVLW	0x03
		MOVWF	cyklus_int
Delenie_8
		BCF		STATUS, C
		RRF		velky_krok+1, F
		RRF		velky_krok, F
		DECFSZ	cyklus_int, F
		GOTO	Delenie_8

		BCF		STATUS, C
		RRF		velky_krok+1, W
		MOVWF	maly_krok+1
		RRF		velky_krok, W
		MOVWF	maly_krok

		MOVLW	0x1B ;27 taktov
		SUBWF	velky_krok, F
		BTFSC	STATUS, C
		MOVLW	0x00
		BTFSS	STATUS, C
		MOVLW	0x01
		SUBWF	velky_krok+1, F

		MOVF	velky_krok, W
		ADDWF	maly_krok, F
		BTFSC	STATUS, C
		INCF	velky_krok+1, W
		BTFSS	STATUS, C
		MOVF	velky_krok+1, W
		ADDWF	maly_krok+1, F

		COMF	maly_krok, F
		COMF	maly_krok+1, F

		COMF	velky_krok, F
		COMF	velky_krok+1, F
		
		BSF		rs232, KAL_OK
		BSF		rs232, MERAM_KAL

		GOTO	Koniec_rc		
Preskoc_vypocet_kroku
		
Koniec_rc
RETURN


;Prerusenie_EEPROM
;		BCF		PIR1, EEIF
;		BANKSEL	rs232
;		BSF		rs232, EE_WR_OK
;RETURN


Prerusenie_TMR0
		BCF		INTCON, T0IF
		BSF		stav, TMR0_TICK
		BCF		INTCON, T0IE
RETURN


Prerusenie_TMR1
;		BSF		stav, TMR1_TICK
		BTFSS	rs232, DATA_GO
		GOTO	TMR1_end
		CLRF	cakatel+1
		BCF		STATUS, C
		BTFSS	PORTB, RB0_RX
		BSF		STATUS, C
		RRF		rc_bajt, F
		MOVF	velky_krok, W
		MOVWF	TMR1L
		MOVF	velky_krok+1, W
		MOVWF	TMR1H
		DECFSZ	posun_bitov, F
		GOTO	TMR1_end
		BCF		T1CON, TMR1ON
		MOVF	rc_bajt, W
		MOVWF	rc_bajt2
		;MOVWF	vykresli
		;CALL	Vykresli_LED
		BSF		stav, SPRACUJ
		;BCF		rs232, DATA_GO
		;CLRF	rs232
		BCF		rs232, DATA_GO
		BCF		INTCON, INTF
		BSF		INTCON, INTE
TMR1_end
		BCF		PIR1, TMR1IF
RETURN



EE        CODE    0x2100
        DE    	0x00, 0x00, 0x00, 0x00

	;	DE		0x01, 0x01 ;zapis bit na PORT ;4
;		DE		0x31, 0x50 ;rotuj doprava predel 2
;		DE		0x10, 0x08
;		DE		0x51, 0x50 ;rotuj dolava predel 2
;		DE		0x10, 0x04
;		DE		0x84, 0xFF ;cyklus 255 krat	  ;16

;		DE		0x01, 0x01 ;zapis bit na PORT ;4
;		DE		0x03, 0x01 ;zapis bit na PORT ;6
;		DE		0x02, 0x01 ;zapis bit na PORT ;8
;		DE		0x06, 0x01 ;zapis bit na PORT ;10
;		DE		0x04, 0x01 ;zapis bit na PORT ;12
;		DE		0x05, 0x01 ;zapis bit na PORT ;14
;		DE		0x84, 0x01 ;cyklus 200 krat	  ;16
;		DE		0x84, 0x01 ;cyklus 200 krat	  ;18
;		DE		0x84, 0x01 ;cyklus 200 krat	  ;20
;		DE		0x84, 0x01 ;cyklus 200 krat	  ;22
;		DE		0x84, 0x01 ;cyklus 200 krat	  ;24
;		DE		0x84, 0x01 ;cyklus 200 krat	  ;26
;		DE		0x84, 0x01 ;cyklus 200 krat	  ;28
;		DE		0x84, 0x01 ;cyklus 200 krat	  ;30

;		DE		0x01, 0x05 ;zapis bit na PORT ;18
;		DE		0x03, 0x05 ;zapis bit na PORT ;20
;		DE		0x02, 0x05 ;zapis bit na PORT ;22
;		DE		0x06, 0x05 ;zapis bit na PORT ;24
;		DE		0x04, 0x05 ;zapis bit na PORT ;26
;		DE		0x05, 0x05 ;zapis bit na PORT ;28
;		DE		0x92, 0xC8 ;cyklus 200 krat	  ;30
;		DE		0x01, 0x03 ;zapis bit na PORT ;32
;		DE		0x03, 0x03 ;zapis bit na PORT ;34
;		DE		0x02, 0x03 ;zapis bit na PORT ;36
;		DE		0x06, 0x03 ;zapis bit na PORT ;38
;		DE		0x04, 0x03 ;zapis bit na PORT ;40
;		DE		0x05, 0x03 ;zapis bit na PORT ;42
;		DE		0xA0, 0xFF ;cyklus 255 krat	  ;44
;		DE		0x01, 0x02 ;zapis bit na PORT ;46
;		DE		0x03, 0x02 ;zapis bit na PORT ;48
;		DE		0x02, 0x02 ;zapis bit na PORT ;50
;		DE		0x06, 0x02 ;zapis bit na PORT ;52
;		DE		0x04, 0x02 ;zapis bit na PORT ;54
;		DE		0x05, 0x02 ;zapis bit na PORT ;56
;		DE		0xAE, 0xFF ;cyklus 255 krat	  ;58
;		DE		0x01, 0x01 ;zapis bit na PORT ;60
;		DE		0x03, 0x01 ;zapis bit na PORT ;62
;		DE		0x02, 0x01 ;zapis bit na PORT ;64
;		DE		0x06, 0x01 ;zapis bit na PORT ;66
;		DE		0x04, 0x01 ;zapis bit na PORT ;68
;		DE		0x05, 0x01 ;zapis bit na PORT ;70
;		DE		0xBC, 0xFF ;cyklus 255 krat	  ;72
;		DE		0x01, 0x00 ;zapis bit na PORT ;74
;		DE		0x03, 0x00 ;zapis bit na PORT ;76
;		DE		0x02, 0x00 ;zapis bit na PORT ;78
;		DE		0x06, 0x00 ;zapis bit na PORT ;80
;		DE		0x04, 0x00 ;zapis bit na PORT ;82
;		DE		0x05, 0x00 ;zapis bit na PORT ;84
;		DE		0xCA, 0xFF ;cyklus 255 krat	  ;86
;		DE		0xCA, 0xFF ;cyklus 255 krat	  ;88
;		DE		0xCA, 0xFF ;cyklus 255 krat	  ;90
	

;		DE		0x10, 0x08 ;vetvenie digi AND
;		DE		0x0D, 0x0A ;konfiguracia 
;		DE		0x01, 0x01 ;zapis bit na PORT
;		DE		0x40, 0x01 ;rotuj dolava
;		DE		0x22, 0x01 ;rotuj 2x do prava
;		DE		0x88, 0x02 ; 2 krat opakuj
;		DE		0x02, 0x01 ;zapis bit
;		DE		0x86, 0x03 ; 3 krat opakuj
;		DE		0x40, 0x01 ;rotuj dolava
;		DE		0x86, 0x02 ; 2 krat opakuj
;		DE		0x03, 0x01 ;zapis bit
;		DE		0x86, 0x02 ; 2 krat opakuj
;		DE		0x05, 0x01 ;zapis bit
;		DE		0x86, 0x02 ; 2 krat opakuj
;		DE		0x22, 0x01 ;rotuj 2x do prava
;		DE		0x86, 0x02 ; 2 krat opakuj
;		DE		0x40, 0x01 ;rotuj dolava
;		DE		0x86, 0x02 ; 2 krat opakuj
;		DE		0x22, 0x01 ;rotuj 2x do prava
;		DE		0x86, 0x02 ; 2 krat opakuj
;		DE		0x40, 0x01 ;rotuj dolava
;		DE		0x86, 0x02 ; 2 krat opakuj

;		DE		0x23, 0x03

;posun o 3x30 stupnov
;		DE		0x01, 0x0A
;		DE		0x02, 0x0A
;		DE		0x04, 0x0A
;cyklus
;		DE		0x84, 0x03
;posun o 3x15 stupnov 
;		DE		0x01, 0x03	
;		DE		0x03, 0x03
;		DE		0x02, 0x03
;		DE		0x06, 0x03
;		DE		0x04, 0x03
;		DE		0x05, 0x03
;cyklus 
;		DE		0x8C, 0x04
;velky cyklus
;		DE		0x84, 0x0A 

        END                       ; directive 'end of program'

